#!/usr/bin/perl

use strict;
use warnings;

use Test::More;

BEGIN
{
    my $str = "use Test::Strict;";

    ## no critic (Eval)
    eval($str);
    ## use critic

    plan skip_all => "Skipping as Test::Strict isn't installed"
      if ($@);
}

all_perl_files_ok();
